#include <iostream>
#include <cassert>
#include "stack0.h"

// (1) A has size sz and sz = 2^i for some i.
// (2) -1 <= tp < sz.
// (3) The content of the stack is A[0]A[1]...A[tp] with A[tp] 
//     being the top of the stack.
// (4) The stack is empty iff tp = -1.

stack::stack()
{ 
  A  = new int[2];    // we start with an array of 2 elements
  sz =  2;
  tp = -1;
}

stack::~stack()
{
  delete[] A;         // sz and tp are automatically deleted
}


bool
stack::empty() const 
{ 
  return (tp == -1) ? true : false; 
}


void 
stack::push(int x) 
{
  if (tp+1 == sz)
  { 
    // we double the size of the array A 
    int* B = A;
    sz *= 2;
    A = new int[sz];
    for(int i=0; i <= tp; i++) 
    { A[i] = B[i]; }
    // the old version of A is no longer needed
    delete[] B; 
  }
  // there is room for the new element
  A[++tp] = x;
}


void
stack::pop()
{
  assert( !empty() );
  tp--;
}


int 
stack::top() const
{ 
  assert( !empty() );
  return A[tp];
}

// The running time of all operations except push is clearly bounded by
// a (small) constant. So let us show that the total time spent in n push
// operations is O(n). Since only push operations increase the size of the
// stack the maximal size of the stack is bounded by n, and hence sz = 2*2^i
// for some i and sz <= 2*n after the n push operations. It takes time 
// proportional to 2^j to increase the size from 2*2^j to 2*2^{j+1}. 
// So the total time spent in reallocating the array A is proportional to
// \sum_{j=0}^{i-1} 2^j <= 2^i <= 2*n.
// The space requirement is proportional to the current value of sz. This is bounded
// by twice the maximal number of elements ever in the stack
