#include <iostream>
#include <cassert>
#include "stack_t.h"

// (1) A has size sz and sz = 2^i for some i.
// (2) -1 <= tp < sz.
// (3) The content of the stack is A[0]A[1]...A[tp] with A[tp] 
//     being the top of the stack.
// (4) The stack is empty iff tp = -1.

template < typename T>
stack<T>::stack()
{ 
  A  = new T[2];
  sz =  2;
  tp = -1;
}

template < typename T>
stack<T>::stack(const stack<T>& S)
{ 
  sz = S.sz;
  tp = S.tp;
  A = new T[sz];
  for (int i = 0; i <= tp; i++) 
  { A[i] = S.A[i]; }
}

template < typename T>
stack<T>::~stack()
{
  delete[] A;         // sz and tp are automatically deleted
}


template < typename T>
stack<T>& 
stack<T>::operator=(const stack<T>& S)
{ 
  if (&S != this)      //  S == S ?
  { 
    if (A) 
    { delete[] A; }
    sz = S.sz;
    tp = S.tp;
    A = new T[sz];
    for (int i = 0; i <= tp; i++) 
    { A[i] = S.A[i]; }
  }
  return (*this);
}


template < typename T>
bool
stack<T>::empty() const 
{ 
  return (tp == -1) ? true : false; 
}


template < typename T>
void 
stack<T>::push(const T& x) 
{
  if (tp+1 == sz)
  { 
    T* B = A;
    sz *= 2;
    A = new T[sz];
    for(int i=0; i <= tp; i++) 
    { A[i] = B[i]; }
    delete[] B; 
  }
  A[++tp] = x;
}


template < typename T>
void
stack<T>::pop()
{
  assert( !empty() );
  tp--;
}


template < typename T>
T 
stack<T>::top() const
{ 
  assert( !empty() );
  return A[tp];
}

